// by LordSkelethom
#include <sstream>
#include <fstream>
#include <vector>
#include <engine/shared/config.h>
#include <game/mapitems.h>
#include <game/server/entities/character.h>
#include <game/server/player.h>
#include <game/server/gamecontext.h>
#include "hprace.h"

CGameControllerHPRace::CGameControllerHPRace(class CGameContext *pGameServer)
: IGameController(pGameServer)
{
	m_pGameType = "HpRACE";
	
	g_Config.m_SvHammerPower = 5;
	g_Config.m_SvTeleport = 1;
	GameServer()->Tuning()->m_PlayerCollision = 1;
	GameServer()->Tuning()->m_PlayerHooking = 1;

	score.init(pGameServer);
	
	for(int i=0;i<(MAX_CLIENTS/2);i++)
		hpTeams[i] = 0;
}

int CGameControllerHPRace::OnCharacterDeath(class CCharacter *pVictim, class CPlayer *pKiller, int Weapon)
{
	return 0;
}

void CGameControllerHPRace::Tick()
{
	IGameController::Tick();
	
	DoRaceTimeCheck();
}

int CGameControllerHPRace::CreateHPTeam(int cid, int cid2)
{
	int id = -1;
	std::vector<int> randt;
	for(int i=0;i<(MAX_CLIENTS/2);i++)
	{
		if(!hpTeams[i])
			randt.push_back(i);
	}
	
	while(id==-1)//choose a random color
	{
		//dbg_msg("debug","randt.size='%d'",randt.size());
		for(int i=0; i<10; i++)
		{
			id = (int) ((float) rand() / RAND_MAX * (randt.size()));
			//dbg_msg("debug","id='%d'",id);
		}
	}
	
	id = randt[id];
	
	if(hpTeams[id])
	{
		dbg_msg("BUG","id='%d'",id);
		return 0;
	}
	
	char ip1[16];
	char ip2[16];
	Server()->GetIP(cid, ip1, sizeof(ip1));
	Server()->GetIP(cid2, ip2, sizeof(ip2));
	hpTeams[id] = new HPRACE_TEAM_SCORE(Server()->ClientName(cid), Server()->ClientName(cid2), 0.0f, ip1, ip2, id);
	
	GameServer()->m_apPlayers[cid]->hprace_team = id;
	GameServer()->m_apPlayers[cid2]->hprace_team = id;
	GameServer()->m_apPlayers[cid]->asked = -1;
	GameServer()->m_apPlayers[cid2]->asked = -1;
	
	GameServer()->m_apPlayers[cid]->KillCharacter(-1);
	GameServer()->m_apPlayers[cid2]->KillCharacter(-1);
	
	if(GameServer()->m_apPlayers[cid]->GetTeam() == -1)
		GameServer()->m_apPlayers[cid]->SetTeam(0);
	if(GameServer()->m_apPlayers[cid2]->GetTeam() == -1)
		GameServer()->m_apPlayers[cid2]->SetTeam(0);
	
	dbg_msg("hprace","%s & %s are now partners ; team='%d'",Server()->ClientName(cid),Server()->ClientName(cid2),id);
	
	return id;
}

void CGameControllerHPRace::DeleteHPTeam(int id)
{
	if(id>-1 && hpTeams[id])
	{
		delete hpTeams[id];
		hpTeams[id] = 0;
	}
}

HPRACE_TEAM_SCORE *CGameControllerHPRace::GetHPTeam(int id)
{
	if(hpTeams[id])
		return hpTeams[id];
	return 0;
}

void CGameControllerHPRace::SetHPTeamScore(int id, float score)
{
	if(id>-1 && hpTeams[id])
		hpTeams[id]->score = score;
}

float CGameControllerHPRace::GetHPTeamScore(int id)
{
	if(id>-1 && hpTeams[id])
		return hpTeams[id]->score;
	return -1;
}

bool CGameControllerHPRace::CanSpawn(int Team, vec2 *pPos)
{
	CSpawnEval eval;
	
	eval.m_FriendlyTeam = Team;
	
	// try first try own team spawn, then normal spawn and then enemy
	EvaluateSpawnType(&eval, 1+(Team&1));
	if(!eval.m_Got)
	{
		EvaluateSpawnType(&eval, 0);
		if(!eval.m_Got)
			EvaluateSpawnType(&eval, 1+((Team+1)&1));
	}
	
	*pPos = eval.m_Pos;
	return eval.m_Got;
}

bool CGameControllerHPRace::IsRace() const
{
	return true;
}

bool CGameControllerHPRace::IsHPRace() const
{
	return true;
}
