// by LordSkelethom
#include <string.h>
#include <sstream>
#include <fstream>
#include <list>
#include <string>
#include <engine/shared/config.h>
#include "gamecontext.h"
#include "hprace_score.h"

HPRACE_TEAM_SCORE::HPRACE_TEAM_SCORE(const char *name, const char *name2, float score, const char *ip, const char *ip2, int id)
{
	str_copy(this->name, name, sizeof(this->name));
	str_copy(this->name2, name2, sizeof(this->name2));
	this->score = score;
	str_copy(this->ip, ip, sizeof(this->ip));
	str_copy(this->ip2, ip2, sizeof(this->ip2));
	this->id = id;
}

int HPRACE_TEAM_SCORE::get_color()
{
	switch(id)
	{
		case 0:
		return 16776990;
		case 2:
		return 2359089;
		case 1:
		return 1113860;
		case 4:
		return 4521776;
		case 3:
		return 8322816;
		case 6:
		return 11057664;
		case 5:
		return 12836395;
		case 7:
		return 15073024;
		default:
		return 16777215;
	}
}

static LOCK score_lock = 0;

std::list<HPRACE_TEAM_SCORE> top5;

HPRACE_SCORE::HPRACE_SCORE()
{
	if(score_lock == 0)
		score_lock = lock_create();
	load();
}

void HPRACE_SCORE::init(class CGameContext *pGameServer)
{
	m_pGameServer = pGameServer;
	m_pServer = m_pGameServer->Server();
}

std::string save_hpfile()
{
	std::ostringstream oss;
	oss << g_Config.m_SvMap << "_HpRecord.dtb";
	return oss.str();
}

static void save_hpscore_thread(void *)
{
	lock_wait(score_lock);
	std::fstream f;
	f.open(save_hpfile().c_str(), std::ios::out);
	if(!f.fail())
	{
		int t = 0;
		for(std::list<HPRACE_TEAM_SCORE>::iterator i = top5.begin(); i != top5.end(); i++)
		{
			f << i->name << std::endl << i->name2 << std::endl << i->score << std::endl  << i->ip << std::endl << i->ip2 << std::endl;
			t++;
			if(t%50 == 0)
				thread_sleep(1);
		}
	}
	f.close();
	lock_release(score_lock);
}

void HPRACE_SCORE::save()
{
	void *save_thread = thread_create(save_hpscore_thread, 0);
#if defined(CONF_FAMILY_UNIX)
	pthread_detach((pthread_t)save_thread);
#endif
}

void HPRACE_SCORE::load()
{
	lock_wait(score_lock);
	std::fstream f;
	f.open(save_hpfile().c_str(), std::ios::in);
	top5.clear();
	while (!f.eof() && !f.fail())
	{
		std::string tmpname, tmpname2, tmpscore, tmpip, tmpip2;
		std::getline(f, tmpname);
		if(!f.eof() && tmpname != "")
		{
			std::getline(f, tmpname2);
			std::getline(f, tmpscore);
			std::getline(f, tmpip);
			std::getline(f, tmpip2);
			top5.push_back(*new HPRACE_TEAM_SCORE(tmpname.c_str(), tmpname2.c_str(), atof(tmpscore.c_str()), tmpip.c_str(), tmpip2.c_str()));
		}
	}
	f.close();
	lock_release(score_lock);
}

HPRACE_TEAM_SCORE *HPRACE_SCORE::search_score(int id, bool score_ip, int *position)
{
	char ip[16];
	Server()->GetIP(id, ip, sizeof(ip));
	
	int pos = 1;
	for(std::list<HPRACE_TEAM_SCORE>::iterator i = top5.begin(); i != top5.end(); i++)
	{
		if((!strcmp(i->ip, ip) || !strcmp(i->ip2, ip)) && g_Config.m_SvScoreIP && score_ip)
		{
			if(position)
				*position = pos;
			return & (*i);
		}
		pos++;
	}
	
	return search_name(Server()->ClientName(id), position, 0);
}

HPRACE_TEAM_SCORE *HPRACE_SCORE::search_name(const char *name, int *position, bool nocase)
{
	HPRACE_TEAM_SCORE *player = 0;
	int pos = 1;
	int found = 0;
	for (std::list<HPRACE_TEAM_SCORE>::iterator i = top5.begin(); i != top5.end(); i++)
	{
		if(str_find_nocase(i->name, name) || str_find_nocase(i->name2, name))
		{
			if(position)
				*position = pos;
			if(nocase)
			{
				found++;
				return & (*i);
			}
			if(!strcmp(i->name, name) || !strcmp(i->name2, name))
				return & (*i);
		}
		pos++;
	}
	return player;
}

HPRACE_TEAM_SCORE *HPRACE_SCORE::search_team(const char *name, const char *name2, int *position)
{
	HPRACE_TEAM_SCORE *team = 0;
	int pos = 1;
	int found = 0;
	for (std::list<HPRACE_TEAM_SCORE>::iterator i = top5.begin(); i != top5.end(); i++)
	{
		if((str_find_nocase(i->name, name) || str_find_nocase(i->name2, name)) 
		&& (str_find_nocase(i->name, name2) || str_find_nocase(i->name2, name2)))
		{
			if(position)
				*position = pos;
			if((!strcmp(i->name, name) || !strcmp(i->name2, name))
			&& (!strcmp(i->name, name2) || !strcmp(i->name2, name2)))
				return & (*i);
		}
		pos++;
	}
	if(found > 1)
	{
		if(position)
			*position = -1;
		return 0;
	}
	return team;
}

void HPRACE_SCORE::parseTeam(HPRACE_TEAM_SCORE *_team)
{
	lock_wait(score_lock);
	HPRACE_TEAM_SCORE *team = search_team(_team->name, _team->name2, NULL);
	if(team)
	{
		if(team->score > _team->score)
			team->score = _team->score;
	}
	else
		top5.push_back(*_team);
	
	top5.sort();
	lock_release(score_lock);
	save();
}

void HPRACE_SCORE::initTeam(int id, int id2)
{
	char ip[16];
	char ip2[16];
	Server()->GetIP(id, ip, sizeof(ip));
	Server()->GetIP(id2, ip2, sizeof(ip2));
	HPRACE_TEAM_SCORE *team = search_score(id, 0, 0);
	if(team)
	{
		lock_wait(score_lock);
		str_copy(team->ip, ip, sizeof(team->ip));
		str_copy(team->ip2, ip2, sizeof(team->ip));
		lock_release(score_lock);
		save();
	}
}

void HPRACE_SCORE::top5_draw(int id, int debut)
{
	int pos = 1;
	char buf[512];
	GameServer()->SendChatTarget(id, "----------- Top 5 -----------");
	for (std::list<HPRACE_TEAM_SCORE>::iterator i = top5.begin(); i != top5.end() && pos <= 5+debut; i++)
	{
		if(pos >= debut)
		{
			str_format(buf, sizeof(buf), "%d. %s & %s - Time: %d minute(s) %5.3f second(s)", pos, i->name, i->name2
			, (int) i->score/60, i->score-((int)i->score/60*60));
			GameServer()->SendChatTarget(id, buf);
		}
		pos++;
	}
	GameServer()->SendChatTarget(id, "------------------------------");
}
